/*
Copyright 2016 The Kubernetes Authors All rights reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package main

import (
	"io/ioutil"
	"os"
	"path/filepath"
	"testing"

	"k8s.io/helm/pkg/chartutil"
	"k8s.io/helm/pkg/hapi/chart"
	"k8s.io/helm/pkg/hapi/release"
	"k8s.io/helm/pkg/helm"
)

func TestUpgradeCmd(t *testing.T) {
	tmpChart, _ := ioutil.TempDir("testdata", "tmp")
	defer os.RemoveAll(tmpChart)
	cfile := &chart.Metadata{
		Name:        "testUpgradeChart",
		Description: "A Helm chart for Kubernetes",
		Version:     "0.1.0",
	}
	chartPath, err := chartutil.Create(cfile, tmpChart)
	if err != nil {
		t.Errorf("Error creating chart for upgrade: %v", err)
	}
	ch, _ := chartutil.Load(chartPath)
	_ = helm.ReleaseMock(&helm.MockReleaseOptions{
		Name:  "funny-bunny",
		Chart: ch,
	})

	// update chart version
	cfile = &chart.Metadata{
		Name:        "testUpgradeChart",
		Description: "A Helm chart for Kubernetes",
		Version:     "0.1.2",
	}

	chartPath, err = chartutil.Create(cfile, tmpChart)
	if err != nil {
		t.Errorf("Error creating chart: %v", err)
	}
	ch, err = chartutil.Load(chartPath)
	if err != nil {
		t.Errorf("Error loading updated chart: %v", err)
	}

	// update chart version again
	cfile = &chart.Metadata{
		Name:        "testUpgradeChart",
		Description: "A Helm chart for Kubernetes",
		Version:     "0.1.3",
	}

	chartPath, err = chartutil.Create(cfile, tmpChart)
	if err != nil {
		t.Errorf("Error creating chart: %v", err)
	}
	var ch2 *chart.Chart
	ch2, err = chartutil.Load(chartPath)
	if err != nil {
		t.Errorf("Error loading updated chart: %v", err)
	}

	originalDepsPath := filepath.Join("testdata/testcharts/reqtest")
	missingDepsPath := filepath.Join("testdata/testcharts/chart-missing-deps")
	badDepsPath := filepath.Join("testdata/testcharts/chart-bad-requirements")
	var ch3 *chart.Chart
	ch3, err = chartutil.Load(originalDepsPath)
	if err != nil {
		t.Errorf("Error loading chart with missing dependencies: %v", err)
	}

	relMock := func(n string, v int, ch *chart.Chart) *release.Release {
		return helm.ReleaseMock(&helm.MockReleaseOptions{Name: n, Version: v, Chart: ch})
	}

	tests := []releaseCase{
		{
			name:    "upgrade a release",
			cmd:     "upgrade funny-bunny " + chartPath,
			resp:    relMock("funny-bunny", 2, ch),
			matches: "Release \"funny-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("funny-bunny", 2, ch)},
		},
		{
			name:    "upgrade a release with timeout",
			cmd:     "upgrade funny-bunny --timeout 120 " + chartPath,
			resp:    relMock("funny-bunny", 3, ch2),
			matches: "Release \"funny-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("funny-bunny", 3, ch2)},
		},
		{
			name:    "upgrade a release with --reset-values",
			cmd:     "upgrade funny-bunny --reset-values " + chartPath,
			resp:    relMock("funny-bunny", 4, ch2),
			matches: "Release \"funny-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("funny-bunny", 4, ch2)},
		},
		{
			name:    "upgrade a release with --reuse-values",
			cmd:     "upgrade funny-bunny --reuse-values " + chartPath,
			resp:    relMock("funny-bunny", 5, ch2),
			matches: "Release \"funny-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("funny-bunny", 5, ch2)},
		},
		{
			name:    "install a release with 'upgrade --install'",
			cmd:     "upgrade zany-bunny -i " + chartPath,
			resp:    relMock("zany-bunny", 1, ch),
			matches: "Release \"zany-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("zany-bunny", 1, ch)},
		},
		{
			name:    "install a release with 'upgrade --install' and timeout",
			cmd:     "upgrade crazy-bunny -i --timeout 120 " + chartPath,
			resp:    relMock("crazy-bunny", 1, ch),
			matches: "Release \"crazy-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("crazy-bunny", 1, ch)},
		},
		{
			name:    "upgrade a release with wait",
			cmd:     "upgrade crazy-bunny --wait " + chartPath,
			resp:    relMock("crazy-bunny", 2, ch2),
			matches: "Release \"crazy-bunny\" has been upgraded. Happy Helming!\n",
			rels:    []*release.Release{relMock("crazy-bunny", 2, ch2)},
		},
		{
			name:      "upgrade a release with missing dependencies",
			cmd:       "upgrade bonkers-bunny" + missingDepsPath,
			resp:      relMock("bonkers-bunny", 1, ch3),
			wantError: true,
		},
		{
			name:      "upgrade a release with bad dependencies",
			cmd:       "upgrade bonkers-bunny " + badDepsPath,
			resp:      relMock("bonkers-bunny", 1, ch3),
			wantError: true,
		},
	}
	testReleaseCmd(t, tests)

}
